import React, { Component } from 'react';
import {
  Platform,
  StyleSheet,
  Text,
  View,
    Alert,
    Linking,
    ImageBackground
} from 'react-native';
import uniqueId from '../utils/id-assigner';
import formatUnit from '../utils/numberFormat';
import ShadowView from '../shadow_view';
import hexConverter from '../utils/hex-format';


function layoutStyle(attr, style = {}) {
    if(attr.layout_width === 'wrap_content')
        style.alignSelf = 'flex-start';
    else if(attr.layout_width)
        style.width = attr.layout_width;
    else
        style.width = '100%';

    if(attr.layout_height === 'match_parent')
        style.height = '100%';
    else if(attr.layout_height != null && attr.layout_height !== 'wrap_content')
        style.height = attr.layout_height;

    if(attr.min_width)
        style.minWidth = formatUnit(attr.min_width);

    if(attr.max_width)
        style.maxWidth = formatUnit(attr.max_width);

    if(attr.min_height)
        style.minHeight = formatUnit(attr.min_height);

    if(attr.max_height)
        style.maxHeight = formatUnit(attr.max_height);

    if(attr.layout_top)
        style.top = formatUnit(attr.layout_top);

    if(attr.layout_left)
        style.left = formatUnit(attr.layout_left);

    if(attr.layout_right)
        style.right = formatUnit(attr.layout_right);

    if(attr.layout_bottom) 
        style.bottom = formatUnit(attr.layout_bottom);

    if(attr.layout_margin)
        style.margin = formatUnit(attr.layout_margin);
    else {
        if(attr.layout_margin_top)
            style.marginTop = formatUnit(attr.layout_margin_top);

        if(attr.layout_margin_left)
            style.marginLeft = formatUnit(attr.layout_margin_left);

        if(attr.layout_margin_right)
            style.marginRight = formatUnit(attr.layout_margin_right);

        if(attr.layout_margin_bottom)
            style.marginBottom = formatUnit(attr.layout_margin_bottom);
    }

    if(attr.padding)
        style.padding = formatUnit(attr.padding);
    else {
        if(attr.padding_top)
            style.paddingTop = formatUnit(attr.padding_top);

        if(attr.padding_left)
            style.paddingLeft = formatUnit(attr.padding_left);

        if(attr.padding_right)
            style.paddingRight = formatUnit(attr.padding_right);

        if(attr.padding_bottom)
            style.paddingBottom = formatUnit(attr.padding_bottom);
    }

    if(typeof attr.corner_radius === 'number')
        style.borderRadius = attr.corner_radius;
    else if (typeof attr.corner_radius === 'string' && attr.corner_radius.endsWith('%'))
        style.borderRadius = style.width ? style.width/2 : 1000;
    else {
        if(attr.corner_top_left_radius)
            style.borderTopLeftRadius = formatUnit(attr.corner_top_left_radius);

        if(attr.corner_top_right_radius)
            style.borderTopRightRadius = formatUnit(attr.corner_top_right_radius);

        if(attr.corner_bottom_left_radius)
            style.borderBottomLeftRadius = formatUnit(attr.corner_bottom_left_radius);

        if(attr.corner_bottom_right_radius)
            style.borderBottomRightRadius = formatUnit(attr.corner_bottom_right_radius);
    }

    if(attr.clip_children === false)
        style.overflow = 'visible';
    else
        style.overflow = 'hidden';



    if(attr.background_color)
        style.backgroundColor = attr.background_color;

    return style;
}

function setAttributes(el, attr) {
    let nativeProperties = {};
    if(attr.id)
        nativeProperties.key = attr.id;
    else
        nativeProperties.key = uniqueId();

    if(attr.accessibility_label)
        nativeProperties.accessibilityLabel = attr.accessibility_label;

    if(attr.accessibility_hidden){
        nativeProperties.accessibility_hidden = true;
        nativeProperties.importantForAccessibility = "no-hide-descendants";
    }

    if(attr.link) {
        nativeProperties['onPress'] = () => {
            Linking.canOpenURL(attr.link).then(supported => {
                if(!supported)
                    Alert.alert("Can not open url")
                else
                    Linking.openURL(attr.link);
            })
        }
    }

    return React.cloneElement(el, nativeProperties);
}

function borderStyle(attr, style) {
    let strokeStyles = ['solid', 'dotted', 'dashed'];

    if (attr.stroke_width)
        style.borderWidth = formatUnit(attr.stroke_width);
    if (attr.stroke_color)
        style.borderColor = attr.stroke_color;

    if (strokeStyles.indexOf(attr.stroke_style) !== -1)
        style.borderStyle = attr.stroke_style;
    else if (strokeStyles.indexOf(attr.stroke_top_style) !== -1)
        style.borderStyle = attr.stroke_top_style;
    else if (strokeStyles.indexOf(attr.stroke_left_style) !== -1)
        style.borderStyle = attr.stroke_left_style;
    else if (strokeStyles.indexOf(attr.stroke_right_style) !== -1)
        style.borderStyle = attr.stroke_right_style;
    else if (strokeStyles.indexOf(attr.stroke_bottom_style) !== -1)
        style.borderStyle = attr.stroke_bottom_style;

    if (attr.stroke_top_width)
        style.borderTopWidth = formatUnit(attr.stroke_top_width);
    if (attr.stroke_top_color)
        style.borderTopColor = attr.stroke_top_color;

    if (attr.stroke_left_width)
        style.borderLeftWidth = formatUnit(attr.stroke_left_width);
    if (attr.stroke_left_color)
        style.borderLeftColor = attr.stroke_left_color;

    if (attr.stroke_right_width)
        style.borderRightWidth = formatUnit(attr.stroke_right_width);
    if (attr.stroke_right_color)
        style.borderRightColor = attr.stroke_right_color;

    if (attr.stroke_bottom_width)
        style.borderBottomWidth = formatUnit(attr.stroke_bottom_width);
    if (attr.stroke_bottom_color)
        style.borderBottomColor = attr.stroke_bottom_color;

    return style;
}

function shadowStyle(attr, style) {
    if (attr.shadow_color) {
        let dx = attr.shadow_dx === null ? 0 : attr.shadow_dx;
        let dy = attr.shadow_dy === null ? 0 : attr.shadow_dy;
        style.shadowOffset = {width: dx, height: dy};

        if (typeof attr.shadow_radius === 'number')
            style.shadowRadius = attr.shadow_radius;

        let colorObject = hexConverter(attr.shadow_color);
        style.shadowColor = colorObject.hex;
        style.shadowOpacity = colorObject.opacity === null ? 0 : colorObject.opacity;
    }

}


function transformStyle(attr, style) {
    let transforms = [];
    let translateX = formatUnit(attr.transform_translate_x);

    if (translateX !== 0)
        transforms.push({translateX: translateX});

    let translateY = formatUnit(attr.transform_translate_y);

    if (translateY !== 0)
        transforms.push({translateY: translateY});

    if (typeof attr.transform_rotate === 'number' && attr.transform_rotate !== 0)
        transforms.push({rotate: attr.transform_rotate + 'deg'});

    if (typeof attr.transform_scale === 'number' && attr.transform_scale !== 1)
        transforms.push({scale: attr.transform_scale});

    if(transforms.length)
        style.transform = transforms;
}

function backgroundImage(attr, style, element) {
    if(attr.background_image){
        return (<ImageBackground
            key={uniqueId()}
            source={{ uri: attr.background_image }}
            style={style}
        >
            {element}
        </ImageBackground>);
    }
    return element;
}

class IncitoView {


  constructor(attr, childViews) {
    this.attr = attr;
    this.childViews = childViews;
    this.layoutStyle = layoutStyle(attr);
    this.style = this.customStyle(attr.background_image ? {width: '100%', height: '100%'} : this.layoutStyle);
    borderStyle(attr, this.style);
    transformStyle(attr, this.style);
    shadowStyle(attr, this.layoutStyle);
    }


    attributes() {
      return this.attr;
    }

    customStyle(style) {
      return style;
    }

    setAttributes(el, attr) {
      return setAttributes(el, attr);
    }

    render() {
        let element = this.setAttributes(this.createElement(), this.attributes());
        return backgroundImage(this.attributes(), this.layoutStyle, element);
    }

    createElement() {
      if(!this.attributes().shadow_color) {
          return (
          <View style={this.style} >
              {
                  this.childViews.map(x => x.render())
              }
          </View>
      );
      }
      else {
          return (
              <ShadowView style={this.style} >
                  {
                      this.childViews.map(x => x.render())
                  }
              </ShadowView>
          );
      }
    }
}

module.exports = IncitoView
